==> whoami_1.1.20090520-1.tar.gz: whoami-1.1.20090520/src/Main.cpp <==
/*
  This file is part of "WhoAmI" - A "Guess whats on the photo" riddle game.
  Copyright (C) 2007  Tim Teulings

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
*/

#include <cstdlib>

#include <Lum/Base/L10N.h>
#include <Lum/Base/String.h>

#include <Lum/Button.h>
#include <Lum/Dialog.h>
#include <Lum/Panel.h>

#include <Lum/Dlg/About.h>
#include <Lum/Dlg/Msg.h>

#include <Lum/Model/Action.h>

#include <Lum/OS/Main.h>

#include "Control.h"
#include "DownloadImage.h"
#include "DownloadLists.h"
#include "Image.h"
#include "ImageInfo.h"

#include "config.h"

#include <iostream>

static Lum::Def::AppInfo info;

class MainDialog : public Lum::Dialog
{
private:
  Lum::Model::ActionRef newImageAction;
  Lum::Model::ActionRef peekAction;
  Lum::Model::ActionRef blockGroupAction;
  Lum::Model::ActionRef imageInfoAction;
  Lum::Model::ActionRef aboutAction;
  Control               *control;

  std::list<Image*>     images;
  Image*                currentImage;

  DownloadListsTask     downloadListsTask;
  DownloadImageTask     downloadImageTask;

public:
  MainDialog()
  : newImageAction(new Lum::Model::Action),
    peekAction(new Lum::Model::Action),
    blockGroupAction(new Lum::Model::Action),
    imageInfoAction(new Lum::Model::Action),
    aboutAction(new Lum::Model::Action),
    control(NULL),
    currentImage(NULL)
  {
    Observe(newImageAction);
    Observe(peekAction);
    Observe(blockGroupAction);
    Observe(imageInfoAction);
    Observe(aboutAction);
    Observe(GetOpenedAction());
    Observe(GetClosedAction());

    peekAction->Disable();
    blockGroupAction->Disable();
    imageInfoAction->Disable();

    downloadListsTask.SetParent(this);
    downloadImageTask.SetParent(this);

    Observe(downloadListsTask.GetFinishedAction());
    Observe(downloadListsTask.GetAbortedAction());

    Observe(downloadImageTask.GetFinishedAction());
  }

  void PreInit()
  {
    Lum::Panel       *horiz,*vert;

    horiz=Lum::HPanel::Create(true,true);

    control=new Control();
    control->SetFlex(true,true);
    horiz->Add(control);
    // TODO

    horiz->AddSpace();

    vert=Lum::VPanel::Create(false,true);
    vert->Add(Lum::Button::Create(L"_New image",newImageAction,true,false));
    vert->AddSpace();
    vert->Add(Lum::Button::Create(L"_Peek",peekAction,true,false));
    vert->AddSpace();
    vert->Add(Lum::Button::Create(L"_Block group",blockGroupAction,true,false));
    vert->AddSpace();
    vert->Add(Lum::Button::Create(L"_Image info",imageInfoAction,true,false));

    horiz->Add(vert);

    SetMain(horiz);

    Lum::MenuDesc *menu=new Lum::MenuDesc();

    menu
      ->AddMenuItemSub(_ld(menuProject))
        ->AddMenuItemAction(_ld(menuProjectQuit),Lum::OS::qualifierControl,L"q",GetClosedAction());
    menu
      ->AddMenuItemSub(_ld(menuHelp))
        ->AddMenuItemAction(_ld(menuHelpHelp),NULL)
        ->AddMenuItemAction(_ld(menuHelpAbout),aboutAction);

    SetMenu(menu);

    Dialog::PreInit();
  }

  void BlockGroup()
  {
    assert(currentImage!=NULL);

    for (std::list<Image*>::const_iterator iter=images.begin(); iter!=images.end(); ++iter) {
      if (!(*iter)->HasBeenShown()) {
        if (currentImage->GetAuthor()==(*iter)->GetAuthor()) {
          currentImage->SetShown();
          currentImage->SetImage(NULL);
        }
      }
    }
  }

  size_t GetAvailableImages()
  {
    size_t availableImages=0;

    for (std::list<Image*>::const_iterator iter=images.begin(); iter!=images.end(); ++iter) {
      if (!(*iter)->HasBeenShown()) {
        availableImages++;
      }
    }

    return availableImages;
  }

  void SetNewImage()
  {
    control->SetImage(currentImage->GetImage());

    currentImage->SetShown();
    currentImage->SetImage(NULL);

    peekAction->Enable();
    blockGroupAction->Enable();
    imageInfoAction->Enable();
  }

  void ShowNewImage()
  {
    size_t                      pos;
    std::list<Image*>::iterator iter;

    pos=(size_t)(double(GetAvailableImages())*rand()/RAND_MAX);

    iter=images.begin();
    while (true) {
      if (pos==0 && !(*iter)->HasBeenShown()) {
        break;
      }

      if (!(*iter)->HasBeenShown()) {
        pos--;
      }

      iter++;
    }

    currentImage=(*iter);

    assert(!currentImage->HasBeenShown());

    if (!currentImage->HasImage()) {
      downloadImageTask.SetImage(currentImage);
      downloadImageTask.Start();

      return;
    }

    SetNewImage();
  }

  void TriggerDownloadLists()
  {
    downloadListsTask.SetImageList(images);
    downloadListsTask.Start();
  }

  void TriggerNewImage()
  {
    newImageAction->Off();
    newImageAction->Disable();
    newImageAction->On(false);

    if (GetAvailableImages()<10) {
      TriggerDownloadLists();
      return;
    }

    ShowNewImage();

    newImageAction->Off();
    newImageAction->Enable();
    newImageAction->On(false);
  }

  void Resync(Lum::Base::Model *model, const Lum::Base::ResyncMsg& msg)
  {
    if (model==GetOpenedAction() && GetOpenedAction()->IsFinished()) {
    }
    else if (model==GetClosedAction() && GetClosedAction()->IsFinished()) {
    }
    else if (model==newImageAction && newImageAction->IsFinished()) {
      TriggerNewImage();
    }
    else if (model==peekAction && peekAction->IsFinished()) {
      control->ToggleShowAll();
    }
    else if (model==blockGroupAction && blockGroupAction->IsFinished()) {
      BlockGroup();
    }
    else if (model==imageInfoAction && imageInfoAction->IsFinished()) {
      ImageInfo *imageInfo;

      imageInfo=new ImageInfo(currentImage);
      imageInfo->SetParent(this);

      if (imageInfo->Open()) {
        imageInfo->EventLoop();
        imageInfo->Close();

        delete imageInfo;
      }
    }
    else if (model==aboutAction && aboutAction->IsFinished()) {
      Lum::Dlg::About::Show(this,info);
    }
    else if (model==downloadListsTask.GetFinishedAction() &&
             downloadListsTask.GetFinishedAction()->IsFinished()) {
      downloadListsTask.GetImageList(images);

      if (downloadListsTask.HasError()) {
        Lum::Dlg::Msg::ShowOk(this,
                              L"Error while downloading RSS feeds!",
                              L"There was an error while downloading the RSS feeds:\n"+
                              downloadListsTask.GetError());
      }
      else {
        if (GetAvailableImages()>0) {
          ShowNewImage();
        }
        else {
          Lum::Dlg::Msg::ShowOk(this,
                                L"No new images available!",
                                L"Sorry, there are currently no new images available!\n"
                                L"Please, try again later!");
        }
      }

      newImageAction->Enable();
    }
    else if (model==downloadImageTask.GetAbortedAction() &&
             downloadImageTask.GetAbortedAction()->IsFinished()) {
      newImageAction->Enable();
    }
    else if (model==downloadImageTask.GetFinishedAction() &&
             downloadImageTask.GetFinishedAction()->IsFinished()) {
      if (downloadImageTask.HasError()) {
        Lum::Dlg::Msg::ShowOk(this,
                              L"Error while downloading image!",
                              L"There was an error during download of image data:\n"+
                              downloadImageTask.GetError());
      }
      else {
        SetNewImage();
      }
    }

    Dialog::Resync(model,msg);
  }
};

class Main : public Lum::OS::MainDialog<MainDialog>
{
public:
  Main()
  {
    info.SetProgram(Lum::Base::StringToWString(PACKAGE_NAME));
    info.SetVersion(Lum::Base::StringToWString(PACKAGE_VERSION));
    info.SetDescription(L"What do you see?");
    info.SetAuthor(L"Tim Teulings");
    info.SetContact(L"Tim Teulings <tim@teulings.org>");
    info.SetCopyright(L"(c) 2007, Tim Teulings");
    info.SetLicense(L"GNU Public License");

    srand(time(NULL));
  }

  bool Prepare()
  {
    return Lum::OS::MainDialog<MainDialog>::Prepare();
  }

  void Cleanup()
  {
    Lum::OS::MainDialog<MainDialog>::Cleanup();
  }
};

LUM_MAIN(Main,Lum::Base::StringToWString(PACKAGE_NAME))

==> whoami_1.1.20091009-1.tar.gz: whoami-1.1.20091009/src/Main.cpp <==
/*
  This file is part of "WhoAmI" - A "Guess whats on the photo" riddle game.
  Copyright (C) 2007  Tim Teulings

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
*/

#include <cstdlib>

#include <Lum/Base/L10N.h>
#include <Lum/Base/String.h>

#include <Lum/Def/Menu.h>

#include <Lum/Button.h>
#include <Lum/Dialog.h>
#include <Lum/Panel.h>

#include <Lum/Dlg/About.h>
#include <Lum/Dlg/Msg.h>

#include <Lum/Model/Action.h>

#include <Lum/OS/Main.h>

#include "Control.h"
#include "DownloadImage.h"
#include "DownloadLists.h"
#include "Image.h"
#include "ImageInfo.h"

#include "config.h"

#include <iostream>

static Lum::Def::AppInfo info;

class MainDialog : public Lum::Dialog
{
private:
  Lum::Model::ActionRef newImageAction;
  Lum::Model::ActionRef peekAction;
  Lum::Model::ActionRef blockGroupAction;
  Lum::Model::ActionRef imageInfoAction;
  Lum::Model::ActionRef aboutAction;
  Control               *control;

  std::list<Image*>     images;
  Image*                currentImage;

  DownloadListsTask     downloadListsTask;
  DownloadImageTask     downloadImageTask;

public:
  MainDialog()
  : newImageAction(new Lum::Model::Action),
    peekAction(new Lum::Model::Action),
    blockGroupAction(new Lum::Model::Action),
    imageInfoAction(new Lum::Model::Action),
    aboutAction(new Lum::Model::Action),
    control(NULL),
    currentImage(NULL)
  {
    Observe(newImageAction);
    Observe(peekAction);
    Observe(blockGroupAction);
    Observe(imageInfoAction);
    Observe(aboutAction);
    Observe(GetOpenedAction());
    Observe(GetClosedAction());

    peekAction->Disable();
    blockGroupAction->Disable();
    imageInfoAction->Disable();

    downloadListsTask.SetParent(this);
    downloadImageTask.SetParent(this);

    Observe(downloadListsTask.GetFinishedAction());
    Observe(downloadListsTask.GetAbortedAction());

    Observe(downloadImageTask.GetFinishedAction());
  }

  void PreInit()
  {
    Lum::Panel       *horiz,*vert;

    horiz=Lum::HPanel::Create(true,true);

    control=new Control();
    control->SetFlex(true,true);
    horiz->Add(control);
    // TODO

    horiz->AddSpace();

    vert=Lum::VPanel::Create(false,true);
    vert->Add(Lum::Button::Create(L"_New image",newImageAction,true,false));
    vert->AddSpace();
    vert->Add(Lum::Button::Create(L"_Peek",peekAction,true,false));
    vert->AddSpace();
    vert->Add(Lum::Button::Create(L"_Block group",blockGroupAction,true,false));
    vert->AddSpace();
    vert->Add(Lum::Button::Create(L"_Image info",imageInfoAction,true,false));

    horiz->Add(vert);

    SetMain(horiz);

  Lum::Def::Menu *menu=Lum::Def::Menu::Create();

  menu
    ->GroupProject()
      ->ActionQuit(GetClosedAction())
    ->End()
    ->GroupHelp()
      //->ActionHelp()
      ->ActionAbout(aboutAction)
    ->End();
    SetMenu(menu);

    Dialog::PreInit();
  }

  void BlockGroup()
  {
    assert(currentImage!=NULL);

    for (std::list<Image*>::const_iterator iter=images.begin(); iter!=images.end(); ++iter) {
      if (!(*iter)->HasBeenShown()) {
        if (currentImage->GetAuthor()==(*iter)->GetAuthor()) {
          currentImage->SetShown();
          currentImage->SetImage(NULL);
        }
      }
    }
  }

  size_t GetAvailableImages()
  {
    size_t availableImages=0;

    for (std::list<Image*>::const_iterator iter=images.begin(); iter!=images.end(); ++iter) {
      if (!(*iter)->HasBeenShown()) {
        availableImages++;
      }
    }

    return availableImages;
  }

  void SetNewImage()
  {
    control->SetImage(currentImage->GetImage());

    currentImage->SetShown();
    currentImage->SetImage(NULL);

    peekAction->Enable();
    blockGroupAction->Enable();
    imageInfoAction->Enable();
  }

  void ShowNewImage()
  {
    size_t                      pos;
    std::list<Image*>::iterator iter;

    pos=(size_t)(double(GetAvailableImages())*rand()/RAND_MAX);

    iter=images.begin();
    while (true) {
      if (pos==0 && !(*iter)->HasBeenShown()) {
        break;
      }

      if (!(*iter)->HasBeenShown()) {
        pos--;
      }

      iter++;
    }

    currentImage=(*iter);

    assert(!currentImage->HasBeenShown());

    if (!currentImage->HasImage()) {
      downloadImageTask.SetImage(currentImage);
      downloadImageTask.Start();

      return;
    }

    SetNewImage();
  }

  void TriggerDownloadLists()
  {
    downloadListsTask.SetImageList(images);
    downloadListsTask.Start();
  }

  void TriggerNewImage()
  {
    newImageAction->Off();
    newImageAction->Disable();
    newImageAction->On(false);

    if (GetAvailableImages()<10) {
      TriggerDownloadLists();
      return;
    }

    ShowNewImage();

    newImageAction->Off();
    newImageAction->Enable();
    newImageAction->On(false);
  }

  void Resync(Lum::Base::Model *model, const Lum::Base::ResyncMsg& msg)
  {
    if (model==GetOpenedAction() && GetOpenedAction()->IsFinished()) {
    }
    else if (model==GetClosedAction() && GetClosedAction()->IsFinished()) {
    }
    else if (model==newImageAction && newImageAction->IsFinished()) {
      TriggerNewImage();
    }
    else if (model==peekAction && peekAction->IsFinished()) {
      control->ToggleShowAll();
    }
    else if (model==blockGroupAction && blockGroupAction->IsFinished()) {
      BlockGroup();
    }
    else if (model==imageInfoAction && imageInfoAction->IsFinished()) {
      ImageInfo *imageInfo;

      imageInfo=new ImageInfo(currentImage);
      imageInfo->SetParent(this);

      if (imageInfo->Open()) {
        imageInfo->EventLoop();
        imageInfo->Close();

        delete imageInfo;
      }
    }
    else if (model==aboutAction && aboutAction->IsFinished()) {
      Lum::Dlg::About::Show(this,info);
    }
    else if (model==downloadListsTask.GetFinishedAction() &&
             downloadListsTask.GetFinishedAction()->IsFinished()) {
      downloadListsTask.GetImageList(images);

      if (downloadListsTask.HasError()) {
        Lum::Dlg::Msg::ShowOk(this,
                              L"Error while downloading RSS feeds!",
                              L"There was an error while downloading the RSS feeds:\n"+
                              downloadListsTask.GetError());
      }
      else {
        if (GetAvailableImages()>0) {
          ShowNewImage();
        }
        else {
          Lum::Dlg::Msg::ShowOk(this,
                                L"No new images available!",
                                L"Sorry, there are currently no new images available!\n"
                                L"Please, try again later!");
        }
      }

      newImageAction->Enable();
    }
    else if (model==downloadImageTask.GetAbortedAction() &&
             downloadImageTask.GetAbortedAction()->IsFinished()) {
      newImageAction->Enable();
    }
    else if (model==downloadImageTask.GetFinishedAction() &&
             downloadImageTask.GetFinishedAction()->IsFinished()) {
      if (downloadImageTask.HasError()) {
        Lum::Dlg::Msg::ShowOk(this,
                              L"Error while downloading image!",
                              L"There was an error during download of image data:\n"+
                              downloadImageTask.GetError());
      }
      else {
        SetNewImage();
      }
    }

    Dialog::Resync(model,msg);
  }
};

class Main : public Lum::OS::MainDialog<MainDialog>
{
public:
  Main()
  {
    info.SetProgram(Lum::Base::StringToWString(PACKAGE_NAME));
    info.SetVersion(Lum::Base::StringToWString(PACKAGE_VERSION));
    info.SetDescription(L"What do you see?");
    info.SetAuthor(L"Tim Teulings");
    info.SetContact(L"Tim Teulings <tim@teulings.org>");
    info.SetCopyright(L"(c) 2007, Tim Teulings");
    info.SetLicense(L"GNU Public License");

    srand(time(NULL));
  }

  bool Prepare()
  {
    return Lum::OS::MainDialog<MainDialog>::Prepare();
  }

  void Cleanup()
  {
    Lum::OS::MainDialog<MainDialog>::Cleanup();
  }
};

LUM_MAIN(Main,Lum::Base::StringToWString(PACKAGE_NAME))
